@@ -18,9 +18,9 @@
 #' All options ending with \code{lasso} have a lasso penalty added to the loss for variable selection. \code{sq_loss_lasso_gam}
 #' and \code{logistic_loss_lasso_gam} first use the lasso to select variables and then fit a generalized additive model
 #' with nonparametric additive terms for each selected variable. \code{sq_loss_gam} involves a squared error loss with a generalized additive model and no variable selection.
-#' \code{sq_loss_gbm} involves a squared error loss with a gradient-boosted decision trees model for the benefit score; this
-#' allows for flexible estimation using machine learning and can be useful when the underlying treatment-covariate interaction
-#' is complex.
+#' \code{sq_loss_xgboost} involves a squared error loss with a gradient-boosted decision trees model using \code{xgboost} for the benefit score; this
+#' allows for flexible estimation using machine learning and can be useful when the underlying treatment-covariate interaction is complex. Must specify
+#' \code{params}, \code{nrounds}, \code{nfold}, and optionally, \code{early_stopping_rounds}; see \code{\link[xgboost]{xgb.train}} for details
 #' \itemize{
 #'     \item{\strong{Continuous Outcomes}}
 #'     \itemize{
@@ -35,7 +35,7 @@
 #'         \item{\code{"owl_logistic_flip_loss_gam"}} { - M(y, v) = |y|log(1 + exp\{-sign(y)v\}) with generalized additive model fit on all variables}
 #'         \item{\code{"owl_logistic_loss_lasso_gam"}} { - M(y, v) = ylog(1 + exp\{-v\}) with variables selected by lasso penalty and generalized additive model fit on the selected variables}
 #'         \item{\code{"owl_logistic_flip_loss_lasso_gam"}} { - M(y, v) = |y|log(1 + exp\{-sign(y)v\}) with variables selected by lasso penalty and generalized additive model fit on the selected variables}
-#'         \item{\code{"sq_loss_gbm"}}{ - M(y, v) = (v - y) ^ 2 with gradient-boosted decision trees model}
+#'         \item{\code{"sq_loss_xgboost"}}{ - M(y, v) = (v - y) ^ 2 with gradient-boosted decision trees model}
 #'     }
 #'     \item{\strong{Binary Outcomes}}
 #'     \itemize{
@@ -43,7 +43,6 @@
 #'         \item{\code{"logistic_loss_lasso"}}{ - M(y, v) = -[yv - log(1 + exp\{-v\})] with with linear model and lasso penalty}
 #'         \item{\code{"logistic_loss_lasso_gam"}}{ - M(y, v) = -[yv - log(1 + exp\{-v\})] with variables selected by lasso penalty and generalized additive model fit on the selected variables}
 #'         \item{\code{"logistic_loss_gam"}}{ - M(y, v) = -[yv - log(1 + exp\{-v\})] with generalized additive model fit on all variables}
-#'         \item{\code{"logistic_loss_gbm"}}{ - M(y, v) = -[yv - log(1 + exp\{-v\})] with gradient-boosted decision trees model}
 #'     }
 #'     \item{\strong{Count Outcomes}}
 #'     \itemize{
@@ -51,12 +50,10 @@
 #'         \item{\code{"poisson_loss_lasso"}}{ - M(y, v) = -[yv - exp(v)] with with linear model and lasso penalty}
 #'         \item{\code{"poisson_loss_lasso_gam"}}{ - M(y, v) = -[yv - exp(v)] with variables selected by lasso penalty and generalized additive model fit on the selected variables}
 #'         \item{\code{"poisson_loss_gam"}}{ - M(y, v) = -[yv - exp(v)] with generalized additive model fit on all variables}
-#'         \item{\code{"poisson_loss_gbm"}}{ - M(y, v) = -[yv - exp(v)] with gradient-boosted decision trees model}
 #'     }
 #'     \item{\strong{Time-to-Event Outcomes}}
 #'     \itemize{
 #'         \item{\code{"cox_loss_lasso"}}{ - M corresponds to the negative partial likelihood of the cox model with linear model and additionally a lasso penalty}
-#'         \item{\code{"cox_loss_gbm"}}{ - M corresponds to the negative partial likelihood of the cox model with gradient-boosted decision trees model}
 #'     }
 #' }
 #' @param method subgroup ID model type. Either the weighting or A-learning method of Chen et al, (2017)
@@ -236,11 +233,11 @@
 #' Journal of Statistical Software 98(5), 1-60. doi:10.18637/jss.v098.i05
 #'
 #' Chen, S., Tian, L., Cai, T. and Yu, M. (2017), A general statistical framework for subgroup identification
-#' and comparative treatment scoring. Biometrics. doi:10.1111/biom.12676 \url{https://onlinelibrary.wiley.com/doi/abs/10.1111/biom.12676}
+#' and comparative treatment scoring. Biometrics. doi:10.1111/biom.12676 \doi{10.1111/biom.12676}
 #'
 #' Xu, Y., Yu, M., Zhao, Y. Q., Li, Q., Wang, S., & Shao, J. (2015),
 #'  Regularized outcome weighted subgroup identification for differential treatment effects. Biometrics, 71(3), 645-653.
-#'  doi: 10.1111/biom.12322 \url{https://onlinelibrary.wiley.com/doi/full/10.1111/biom.12322}
+#'  doi: 10.1111/biom.12322 \doi{10.1111/biom.12322}
 #'
 #'  Zhao, Y., Zeng, D., Rush, A. J., & Kosorok, M. R. (2012),
 #'   Estimating individualized treatment rules using outcome weighted learning.
@@ -332,7 +329,9 @@
 #'                            trt = trt01,
 #'                            propensity.func = prop.func,
 #'                            loss   = "sq_loss_lasso",
-#'                            nfolds = 10)              # option for cv.glmnet
+#'                            # option for cv.glmnet,
+#'                            # better to use 'nfolds=10'
+#'                            nfolds = 3)
 #'
 #' summary(subgrp.model)
 #'
@@ -384,7 +383,9 @@
 #'                            propensity.func = prop.func,
 #'                            augment.func    = augment.func,
 #'                            loss   = "sq_loss_lasso",
-#'                            nfolds = 10)              # option for cv.glmnet
+#'                            # option for cv.glmnet,
+#'                            # better to use 'nfolds=10'
+#'                            nfolds = 3)              # option for cv.glmnet
 #'
 #' summary(subgrp.model.aug)
 #' }
@@ -397,7 +398,7 @@
 #'                            propensity.func = prop.func,
 #'                            loss   = "logistic_loss_lasso",
 #'                            type.measure = "auc",    # option for cv.glmnet
-#'                            nfolds = 5)              # option for cv.glmnet
+#'                            nfolds = 3)              # option for cv.glmnet
 #'
 #' subgrp.model.bin
 #'
@@ -410,7 +411,7 @@
 #'                            propensity.func = prop.func,
 #'                            loss   = "poisson_loss_lasso",
 #'                            type.measure = "mse",    # option for cv.glmnet
-#'                            nfolds = 5)              # option for cv.glmnet
+#'                            nfolds = 3)              # option for cv.glmnet
 #'
 #' subgrp.model.poisson
 #'
@@ -423,7 +424,7 @@
 #'                            trt = trt01,
 #'                            propensity.func = prop.func,
 #'                            loss   = "cox_loss_lasso",
-#'                            nfolds = 5)              # option for cv.glmnet
+#'                            nfolds = 3)              # option for cv.glmnet
 #'
 #' subgrp.model.cox
 #' }
@@ -523,10 +524,7 @@ fit.subgroup <- function(x,
                                         "owl_logistic_flip_loss_gam",
                                         "owl_logistic_loss_lasso_gam",
                                         "owl_logistic_flip_loss_lasso_gam",
-                                        "sq_loss_gbm",
-                                        "poisson_loss_gbm",
-                                        "logistic_loss_gbm",
-                                        "cox_loss_gbm",
+                                        "sq_loss_xgboost",
                                         "custom"),
                          method       = c("weighting", "a_learning"),
                          match.id     = NULL,
@@ -729,11 +727,7 @@ fit.subgroup <- function(x,
                              "owl_logistic_flip_loss_gam"       = "adj",
                              "owl_logistic_loss_lasso_gam"      = "adj",
                              "owl_logistic_flip_loss_lasso_gam" = "adj",
-                             "sq_loss_gbm"                      = "offset",
-                             "poisson_loss_gbm"                     = "offset",
-                             "abs_loss_gbm"                     = "offset",
-                             "logistic_loss_gbm"                = "offset",
-                             "cox_loss_gbm"                     = "offset",
+                             "sq_loss_xgboost"                  = "adj",
                              "custom"                           = "offset_notdots")
 
     if (is.factor(trt))
@@ -801,9 +795,9 @@ fit.subgroup <- function(x,
         refnull <- TRUE
     }
 
-    if (n.trts > 2 & (grepl("_gbm", loss) | grepl("_gam", loss)) )
+    if (n.trts > 2 & (grepl("_xgboost", loss) | grepl("_gam", loss)) )
     {
-        stop("gbm and gam based losses not supported for multiple treatments (number of total treatments > 2)")
+        stop("xgboost and gam based losses not supported for multiple treatments (number of total treatments > 2)")
     }
 
     # Check match.id validity and convert it to a factor, if supplied
@@ -1006,14 +1000,25 @@ fit.subgroup <- function(x,
     if (grepl("owl_", loss))
     {
         x.tilde <- cbind(1, x)
+        trt.multiplier <- rep(1, NROW(x))
     } else
     {
         # construct design matrix to be passed to fitting function
-        x.tilde <- create.design.matrix(x             = x,
-                                        pi.x          = pi.x,
-                                        trt           = trt,
-                                        method        = method,
-                                        reference.trt = reference.trt)
+        design_objects <- create.design.matrix.split(x             = x,
+                                                     pi.x          = pi.x,
+                                                     trt           = trt,
+                                                     method        = method,
+                                                     reference.trt = reference.trt)
+
+        if (grepl("_xgboost", loss) | grepl("_gam", loss))
+        {
+            x.tilde <- design_objects$x
+        } else
+        {
+            x.tilde <- design_objects$trt.multiplier * design_objects$x
+        }
+
+        trt.multiplier <- design_objects$trt.multiplier
     }
 
     # construct observation weight vector
@@ -1119,7 +1124,7 @@ fit.subgroup <- function(x,
         fitted.model <- do.call(fit_fun, c(list(x = x.tilde, trt = trt, n.trts = n.trts,
                                                 y = trt.y, wts = drop(wts) * drop(y.wt),
                                                 family = family, intercept = intercept,
-                                                match.id = match.id, ...), extra.args) )
+                                                match.id = match.id, trt.multiplier = trt.multiplier, ...), extra.args) )
     } else
     {
         if (loss == "custom")
@@ -1159,7 +1164,7 @@ fit.subgroup <- function(x,
 
             fitted.model <- do.call(fit_fun, c(list(x = x.tilde, trt = trt, n.trts = n.trts,
                                                     y = y.adj, wts = wts, family = family,
-                                                    match.id = match.id, ...), extra.args)  )
+                                                    match.id = match.id, trt.multiplier = trt.multiplier, ...), extra.args)  )
         }
     }
 
