@@ -7,9 +7,10 @@ test_that("test that subgroup effect calculations are correct", {
     bene.score <- rnorm(10)
     y <- rnorm(10)
     trt <- c(rep(1, 5), rep(0, 5))
+    pi.x <- c(rep(0.75, 5), rep(0.25, 5))
 
     sub.eff <- subgroup.effects(benefit.scores = bene.score,
-                                y = y, trt = trt)
+                                y = y, trt = trt, pi.x = pi.x)
 
     recom <- 1 * (bene.score > 0)
 
@@ -26,6 +27,8 @@ test_that("test that subgroup effect calculations are correct", {
     expect_equal(overall, sub.eff$overall.subgroup.effect)
 
     expect_error(subgroup.effects(benefit.scores = bene.score,
-                                  y = y, trt = c(rep(1, 4), rep(0, 3), rep(2, 3))))
+                                  y = y, trt = c(rep(1, 4), rep(0, 3), rep(2, 3)),
+                                  pi.x = c(rep(0.33, 4), rep(0.25, 3), rep(0.75, 3))
+                                  ))
 
 })
